/*
 * 这是benblue_computer的PC模块，他为benblue程序提供计数功能。
 * 
 * v0.1
 * 
 * 从计算机主clock 接入Pin 2 ，引发一个中断。
 * 当 J 为高电位时，从总线读入，地址。
 * 当 CE 线为高电位时，计数器+1 
 * 当 OE 线为低电位时，向总线输出数据。
 * ——熊鹰、熊梓墨
 * 2020-4-19
 */

#include <LiquidCrystal.h>

//定义Pin
#define SHIFT_DATA      A5 // 595的移位数据线，对应 595的引脚14号引脚
#define SHIFT_CLK       A3 // 为595提供移位脉冲  595的引脚11号引脚
#define RCLK            A4 // 将寄存器的值放入输出寄存器。595的引脚12引脚
#define J_DATA          4  // 总线数据
#define PL              5  // 串并行控制线，LOW表示并行，HIGH表示串行
#define CP              6  // Jump clock
#define J               A2 // Jump线，从总线中读入数据
#define CE              A1 // CE 计数器加一。 
#define CLOCK           2  // 来源于主机的主脉冲。


//LCD 1602输出控制

#define LCD_RS 7
#define LCD_EN 8
#define LCD_D4 9      
#define LCD_D5 10
#define LCD_D6 11
#define LCD_D7 12


unsigned int address = 0x0000;
unsigned int move_flag = 0x0001;
unsigned int data = 0;
unsigned int ce_value = 0;
unsigned int delta_j = 0;

LiquidCrystal lcd(LCD_RS, LCD_EN, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

void output_595(unsigned int address);     //将数据写入到595中
void read_165(void);                       //将数据从165读入到arduino;
void show_address(unsigned int address);   //显示数据到LCD中;
void com_clock_irq (void);

void com_clock_irq(void){
  ce_value = digitalRead(CE);
  delta_j = digitalRead(J);
  Serial.println(delta_j);
  if (ce_value == 1){
     address++;
     output_595(address);
     show_address(address);
  }
  if (delta_j == 1){
    read_165();
    show_address(address);
    output_595(address);
  }
};

void show_address(unsigned int address){
  unsigned int flag = 0x000f;
  unsigned int index = 0;
  char print_key[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
  char str[16]       = {' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '};
  str[0] = '0';
  str[1] = 'x';
  for (int i = 5 ; i > 1 ; i--){
    index = address & flag;
    str[i] = print_key[index];
    address = address >> 4;
  }
  lcd.noCursor();
  lcd.setCursor(0,1);
  lcd.print(str);
}


void read_165(void){
    Serial.print("data = ");
           digitalWrite(PL,0);
       digitalWrite(PL,1);
       for (int i = 0;i < 16;i++){
           data = digitalRead(J_DATA);
           Serial.print("data = ");
           Serial.println(data);
           if (data == 1){
                address = address >> 1;
                address = 0x8000 | address;
           }
           else{
                address = address >> 1;
           }
           Serial.print("address = ");
           Serial.println(address);
           digitalWrite(CP,1);
           digitalWrite(CP,0);
        }
      Serial.print("address = ");
      Serial.println(address);
      show_address(address);
}

void output_595 (unsigned int address) {
  move_flag = 0x0001;
  for (int i = 0 ; i < 16 ; i++){
    if ((address & move_flag) > 0 ){
      digitalWrite(SHIFT_DATA,1);
    } else {
      digitalWrite(SHIFT_DATA,0);
    }
    digitalWrite(SHIFT_CLK,1);
    digitalWrite(SHIFT_CLK,0);
    move_flag = move_flag << 1;
  }
  digitalWrite (RCLK,1);
  digitalWrite (RCLK,0);
}

void setup() {
  // put your setup code here, to run once:
  lcd.begin(16,2);
  lcd.setCursor(0,0);
  lcd.print("www.benblue.cn");
  
  Serial.begin(57600);
  
  pinMode(SHIFT_DATA,OUTPUT);
  pinMode(SHIFT_CLK,OUTPUT);
  pinMode(RCLK,OUTPUT);

  pinMode(CP,OUTPUT);
  pinMode(PL,OUTPUT);
  pinMode(J_DATA,INPUT);

  pinMode(CE,INPUT);
  pinMode(CLOCK,INPUT);
  pinMode(J, INPUT);
  
  digitalWrite(CP,0);
  digitalWrite(PL,0);
  
  digitalWrite(SHIFT_CLK,LOW);
  digitalWrite(RCLK,LOW);
  output_595(address);
  show_address(address);
  attachInterrupt(digitalPinToInterrupt(CLOCK), com_clock_irq, RISING);
}

void loop() {
  // put your main code here, to run repeatedly:

}
